/***************************************************************************
                          aminoacid.c  -  description
                             -------------------
    begin                : Fri Jun 25 2002
    copyright            : (C) 2003 by Nobuhiro Kuroiwa
    email                : nobu@iastate.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   any later version                                                     *
 *                                                                         *
 ***************************************************************************/
#include "tiams.h"
#include "aminoacid.h"

//---------------------------------------------------------------------------
//	putGroup()
//	put group( CH2, NH3+, etc) of molecule on screen.
//---------------------------------------------------------------------------
void putGroup( WINDOW *pappW, int aanum)
{
	TYPE_NODE			*node;
	WIN_RECT			rect;
	TYPE_COORDINATES	*np			= aa[ aanum].nodePoint;
	int					index_char	= 0;
	int					index_group	= 0;
	int					index_np	= 0;
	
	node = aa[ aanum].nodeAtoms[ index_group];
	
	while( node != NULL)
	{
		//	Assign x and y to rect
		array2rect( &rect, np + index_np, 1);
		//	draw a char in node[]
		while( node[ index_char] != '\0') {
			putAtom(
						pappW,
						&rect.x0,
						&rect.y0,
						node[ index_char++]
					);
		}
		index_char	=	0;
		index_np	+=	2;
		node		=	aa[ aanum].nodeAtoms[ ++index_group];
	}
}

//---------------------------------------------------------------------------
//	putAtom()
//	Put atom or notations( +, -, lower, upper case)
//---------------------------------------------------------------------------
void putAtom( WINDOW *pappW, int *x, int *y, TYPE_NODE anatom)
{
	Access_AMS_Global_Variables;
	WIN_RECT	rect;
	int			add2NextX = 0;
	int			add2NextY = 0;
	int			dx	= 0;
	int			dy	= 0;
	//-----------------------------------------------------------------------
	//	Set location of pen. dx and dy are added    ------------>Y
	//	to current pen position. retSize is         |
	//	increment to next pen position              |X    coord system
	//-----------------------------------------------------------------------
	if( isdigit( anatom) != 0)		//	If digit
	{
		dx = +3;
		dy = -7;
		add2NextY = 4;
	}
	else if( isalpha( anatom) != 0)	//	If alphabet
	{
		dx = -3;
		dy = -2;
		add2NextY = 8;
	}
	else							//	If +-
	{
		dx = 3;
		dy = -12;
		add2NextY = 0;
	}
	//-----------------------------------------------------------------------
	//	Draw a letter on a adjusted position
	//-----------------------------------------------------------------------	
	rect.x0 = *x + dx;
	rect.y0 = *y + dy;
	WinCharXY( pappW, rect.x0, rect.y0, anatom, 1);
	//-----------------------------------------------------------------------
	//	Overwrite set new pen position to draw next letter.
	//-----------------------------------------------------------------------
	*x += add2NextX;
	*y += add2NextY;
}

//---------------------------------------------------------------------------
//	putBond()
//	draw bonding line
//---------------------------------------------------------------------------
void putBond( WINDOW *pappW,  int aanum)
{
	TYPE_COORDINATES	*bond = aa[ aanum].bond;
	int			add2x	= 10;
	int			add2y	= 10;
	int			index	= 0;
	WIN_RECT	rect;
	//-----------------------------------------------------------------------
	//	Drawing Line.
	//	Coordinates are obtained from aa[].bond[] array.
	//-----------------------------------------------------------------------
	while( bond[ index] != END)
	{
		array2rect( &rect, bond+index, 2);	//	load coord to rect
		WinLine( pappW, &rect);				//	draw line
		index += 4;
	}
}

//---------------------------------------------------------------------------
//	array2rect()
//	copy values in array to WIN_RECT.
//	level == 1 :	assign two values from array[0] and array[1]
//					to rect.x0 and rect.y0
//	level == 2 :	assign four values from array[0]-array[3]
//					to rect.x0 rect.y0 rect.x1 and rect.y1
//---------------------------------------------------------------------------
BOOL array2rect( WIN_RECT *prect, TYPE_COORDINATES *array, int level)
{
	WIN_COORDS	*pointer;
	int			counter;
	//-----------------------------------------------------------------------
	//	Get number of loop.
	//-----------------------------------------------------------------------
	level *= 2;
	//-----------------------------------------------------------------------
	//	Check the array is valid or not.
	//-----------------------------------------------------------------------
	for( counter = 0; counter < level; counter++)
		if( array[ counter] == END)		return FALSE;
	//-----------------------------------------------------------------------
	//	Assign values from array to rect
	//-----------------------------------------------------------------------
	pointer = &( prect->x0);
	for( counter = 0; counter < level; counter++)
	{
		*(pointer)	= array[ counter];
		pointer++;
	}
	return TRUE;
}

//---------------------------------------------------------------------------
//	putAminoAcid()
//	Draw bond and group
//---------------------------------------------------------------------------
void putAminoAcid( WINDOW *pappW, int aanum)
{
	int x = 50;
	int y = 50;
	putBond( pappW, aanum);
	putGroup( pappW, aanum);
}

//---------------------------------------------------------------------------
//	checkBorder()
//	check if the number is leagal or not
//---------------------------------------------------------------------------
BOOL checkBorder( int num)
{
	if( 0 <= num && num < 20)	return TRUE;
	else						return FALSE;
}

//---------------------------------------------------------------------------
//	getTrigraph()
//	returns 3-letter expression in buf[]
//---------------------------------------------------------------------------
void	getTrigraph( int num, char *buf)
{
	//	If outof border, return
	if( checkBorder( num) == FALSE)
	{
		buf[0] = '\0';
		return;
	}
	strncpy( buf, aa[ num].tri_graph, 3);
	buf[3] = '\0';
}

//---------------------------------------------------------------------------
//	getName()
//	returns a name of an aminoacid in buf[]
//---------------------------------------------------------------------------
void	getName( int num, char *buf)
{
	//	If outof border, return
	if( checkBorder( num) == FALSE)
	{
		buf[0] = '\0';
		return;
	}
	strcpy( buf, aa[ num].name);
}

//---------------------------------------------------------------------------
//	get1Letter()
//	returns a 1-letter expression of amoinoacid as a char
//---------------------------------------------------------------------------
char get1Letter( int num)
{
	if( checkBorder( num) == FALSE)
		return 'x';
	return aa[ num].mono_graph;
}

//---------------------------------------------------------------------------
//	getMonograph()
//	returns 1-letter expression of amionoacid in buf[]
//---------------------------------------------------------------------------
void	getMonograph( int num, char *buf)
{
	if( checkBorder( num) == FALSE)
	{
		buf[0] = '\0';
		return;
	}
	buf[0] = aa[ num].mono_graph;
	buf[1] = '\0';
}
